import * as plugins from './plugins.js';
import { ServiceWorker } from './classes.serviceworker.js';
import { logger } from './logging.js';

export class NetworkManager {
  public serviceWorkerRef: ServiceWorker;
  public webRequest: plugins.webrequest.WebRequest;
  private isOffline: boolean = false;
  private lastOnlineCheck: number = 0;
  private readonly ONLINE_CHECK_INTERVAL = 30000; // 30 seconds

  public previousState: string;

  constructor(serviceWorkerRefArg: ServiceWorker) {
    this.serviceWorkerRef = serviceWorkerRefArg;
    this.webRequest = new plugins.webrequest.WebRequest();
    
    // Listen for connection changes
    this.getConnection()?.addEventListener('change', () => {
      this.updateConnectionStatus();
    });

    // Listen for online/offline events
    self.addEventListener('online', () => {
      this.isOffline = false;
      logger.log('info', 'Device is now online');
      this.updateConnectionStatus();
    });

    self.addEventListener('offline', () => {
      this.isOffline = true;
      logger.log('warn', 'Device is now offline');
      this.updateConnectionStatus();
    });
  }

  /**
   * gets the connection
   */
  public getConnection() {
    const navigatorLocal: any = self.navigator;
    return navigatorLocal?.connection;
  }

  public getEffectiveType() {
    return this.getConnection()?.effectiveType || '4g';
  }

  public updateConnectionStatus() {
    const currentType = this.getEffectiveType();
    logger.log('info', `Connection type changed from ${this.previousState} to ${currentType}`);
    this.previousState = currentType;
  }

  /**
   * Checks if the device is currently online by attempting to contact the server
   * @returns Promise<boolean> true if online, false if offline
   */
  public async checkOnlineStatus(): Promise<boolean> {
    const now = Date.now();
    // Only check if enough time has passed since last check
    if (now - this.lastOnlineCheck < this.ONLINE_CHECK_INTERVAL) {
      return !this.isOffline;
    }

    try {
      const response = await fetch('/sw-typedrequest', {
        method: 'HEAD',
        cache: 'no-cache'
      });
      this.isOffline = false;
      this.lastOnlineCheck = now;
      return true;
    } catch (error) {
      this.isOffline = true;
      this.lastOnlineCheck = now;
      logger.log('warn', 'Device appears to be offline');
      return false;
    }
  }

  /**
   * Makes a network request with offline handling
   * @param request The request to make
   * @param options Additional options
   * @returns Promise<Response>
   */
  public async makeRequest<T>(request: Request | string, options: {
    timeoutMs?: number;
    retries?: number;
    backoffMs?: number;
  } = {}): Promise<Response> {
    const {
      timeoutMs = 5000,
      retries = 1,
      backoffMs = 1000
    } = options;

    let lastError: Error;
    for (let i = 0; i <= retries; i++) {
      try {
        const isOnline = await this.checkOnlineStatus();
        if (!isOnline) {
          throw new Error('Device is offline');
        }

        const controller = new AbortController();
        const timeoutId = setTimeout(() => controller.abort(), timeoutMs);

        const response = await fetch(request, {
          ...typeof request === 'string' ? {} : request,
          signal: controller.signal
        });

        clearTimeout(timeoutId);
        return response;
      } catch (error) {
        lastError = error;
        if (i < retries) {
          await new Promise(resolve => setTimeout(resolve, backoffMs * (i + 1)));
        }
      }
    }

    throw lastError;
  }
}
